/*#############################################################
Driver name	    : ColorClick_F4.c
Author 					: Grant Phillips
Date Modified   : 16/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the ColorClick 
									Color Sensor Module.

Requirements    : * STM32F4-Discovery Board

Functions				: HMC6352_Init
									HMC6352_Write1Byte
									HMC6352_Write2Bytes
									HMC6352_Read1Byte
									HMC6352_Read2Bytes			
									HMC6352_ReadHeading	
													  
##############################################################*/

#include <stdio.h>

#define	COLOR_TIMEOUT_MAX		0x0000ffff
#define COLOR_ADDRESS				0x52

// TCS3471 registers definition
#define COLOR_ENABLE  			0x80    // Enable status and interrupts
#define COLOR_ATIME   			0x81    // RGBC ADC time
#define COLOR_WTIME   			0x83    // Wait time
#define COLOR_AILTL   			0x84    // RGBC Interrupt low treshold low byte
#define COLOR_AILTH   			0x85    // RGBC Interrupt low treshold high byte
#define COLOR_AIHTL   			0x86    // RGBC Interrupt high treshold low byte
#define COLOR_AIHTH   			0x87    // RGBC Interrupt high treshold high byte
#define COLOR_PERS    			0x8C    // Interrupt persistence filters
#define COLOR_CONFIG  			0x8D    // Configuration
#define COLOR_CONTROL 			0x8F    // Gain control register
#define COLOR_ID      			0x92    // Device ID
#define COLOR_STATUS  			0x93    // Device status
#define COLOR_CDATA   			0x94    // Clear ADC low data register
#define COLOR_CDATAH  			0x95    // Clear ADC high data register
#define COLOR_RDATA   			0x96    // RED ADC low data register
#define COLOR_RDATAH  			0x97    // RED ADC high data register
#define COLOR_GDATA   			0x98    // GREEN ADC low data register
#define COLOR_GDATAH  			0x99    // GREEN ADC high data register
#define COLOR_BDATA   			0x9A    // BLUE ADC low data register
#define COLOR_BDATAH  			0x9B    // BLUE ADC high data register

#define COLOR_GAIN_1X				0x00
#define COLOR_GAIN_4X				0x01
#define COLOR_GAIN_16X			0x02 
#define COLOR_GAIN_60X			0x03



void Color_Init(void);
void Color_SetRGBLeds(uint8_t red, uint8_t green, uint8_t blue);
uint8_t Color_Write1Byte(uint8_t byte0);
uint8_t Color_Write2Bytes(uint8_t byte1, uint8_t byte0);
uint8_t Color_Read1Byte(uint8_t address, uint8_t *ptr);
uint8_t HMC6352_Read2Bytes(uint8_t array[]);
void Color_Enable(void);
void Color_Disable(void);
void Color_SetGain(uint8_t gain);
void Color_SetIntegrationTime(uint8_t ATIMEvalue);
uint16_t Color_ReadCData(void);
uint16_t Color_ReadRData(void);
uint16_t Color_ReadGData(void);
uint16_t Color_ReadBData(void);



/*********************************************************************************************
Function name   : Color_Init
Author 					: Grant Phillips
Date Modified   : 16/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the ColorClick Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void Color_Init(void)
{
	GPIO_InitTypeDef 	GPIO_InitStructure;	
	I2C_InitTypeDef  	I2C_InitStructure;
	
	//========== initialize the GPIO pins for RGB LEDs ==========
	
	/* Enable GPIOD clock */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	
	/* Configure PD8, PD9, PD10 as output push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 |GPIO_Pin_9 | GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
		
	
	//========== initializethe I2C bus ==========
	
	//Enable the I2C periph
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
	
	//enable the peripheral clock for the pins used by PB6 for I2C SCL and PB9 for I2C1_SDL
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

	//This sequence sets up the I2C1SDA and I2C1SCL pins so they work correctly with the I2C1 peripheral
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_9; 	// Pins 6(I2C1_SCL) and 9(I2C1_SDA)
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF; 						
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;				
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;					
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;				
	GPIO_Init(GPIOB, &GPIO_InitStructure);								 
	
	//The I2C1_SCL and I2C1_SDA pins are now connected to their AF so that the I2C1 can take over control of the pins
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_I2C1); 
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_I2C1);

	//Enable the I2C peripheral
	I2C_Cmd(I2C1, ENABLE);

	//Set the I2C structure parameters 
	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_OwnAddress1 = 0x00;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = 3000;

	//Initialize the I2C peripheral with selected parameters 
	I2C_Init(I2C1,&I2C_InitStructure);
	
	
	//========== initializethe the actual ColorClick module ==========
	Delay(100);
	Color_Enable();
	Delay(100);
	Color_SetGain(COLOR_GAIN_4X);						// Set gain x16
  Color_SetIntegrationTime(0x00);					// Set acquisition timer of 700 ms
}



/*********************************************************************************************
Function name   : Color_SetRGBLeds
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Controls the output of the RED, GREEN, and BLUE LEDs of the ColorClick Color 
									Sensor Module.  

Special Note(s) : NONE

Parameters			: red (1=ON; 0=OFF), green (1=ON; 0=OFF), blue (1=ON; 0=OFF)

Return value		: NONE
*********************************************************************************************/
void Color_SetRGBLeds(uint8_t red, uint8_t green, uint8_t blue)
{
	if(red==1)
		GPIO_WriteBit(GPIOD, GPIO_Pin_8, Bit_SET);
	else
		GPIO_WriteBit(GPIOD, GPIO_Pin_8, Bit_RESET);
	if(green==1)
		GPIO_WriteBit(GPIOD, GPIO_Pin_9, Bit_SET);
	else
		GPIO_WriteBit(GPIOD, GPIO_Pin_9, Bit_RESET);
	if(blue==1)
		GPIO_WriteBit(GPIOD, GPIO_Pin_10, Bit_SET);
	else
		GPIO_WriteBit(GPIOD, GPIO_Pin_10, Bit_RESET);
}



/*********************************************************************************************
Function name   : Color_Enable
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Enable the ColorClick Color Sensor Module.  

Special Note(s) : NONE

Parameters			: NONE

Return value		: NONE
*********************************************************************************************/
void Color_Enable(void)
{
	Color_Write2Bytes(COLOR_ENABLE, 0x1B);	//AIEN=1; WEN=1; AEN=1; PON=1
}



/*********************************************************************************************
Function name   : Color_Disable
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Disable the ColorClick Color Sensor Module.  

Special Note(s) : NONE

Parameters			: NONE

Return value		: NONE
*********************************************************************************************/
void Color_Disable(void)
{
	Color_Write2Bytes(COLOR_ENABLE, 0x18);	//AIEN=1; WEN=1; AEN=0; PON=0
}



/*********************************************************************************************
Function name   : Color_SetGain
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Sets the gain for the ColorClick Color Sensor Module.  This simply acts as
									a multiplying factor for the color data.

Special Note(s) : NONE

Parameters			: gain (COLOR_GAIN_1X, COLOR_GAIN_4X, COLOR_GAIN_16X or COLOR_GAIN_60X)

Return value		: NONE
*********************************************************************************************/
void Color_SetGain(uint8_t gain)
{
	Color_Write2Bytes(COLOR_CONTROL, gain);	//gain: X1=0x00; X4=0x01; X16=0x02; X60=0x03
}



/*********************************************************************************************
Function name   : Color_SetIntegrationTime
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Sets the intergration time for the ColorClick Color Sensor Module.

Special Note(s) : NONE

Parameters			: ATIMEvalue - 8-bit value that will be written to the ATIME register

Return value		: NONE
*********************************************************************************************/
void Color_SetIntegrationTime(uint8_t ATIMEvalue)
{
	Color_Write2Bytes(COLOR_ATIME, ATIMEvalue);		//Integration Time = 2.4 ms  (256 - ATIMEvalue)
}



/*********************************************************************************************
Function name   : Color_ReadCData
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the CLEAR color data value from ColorClick Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t Color_ReadCData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	Color_Read1Byte(COLOR_CDATA, &low_byte);			//read low byte of channel
	Color_Read1Byte(COLOR_CDATAH, &high_byte);		//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : Color_ReadRData
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the RED color data value from ColorClick Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t Color_ReadRData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	Color_Read1Byte(COLOR_RDATA, &low_byte);			//read low byte of channel
	Color_Read1Byte(COLOR_RDATAH, &high_byte);		//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : Color_ReadGData
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the GREEN color data value from ColorClick Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t Color_ReadGData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	Color_Read1Byte(COLOR_GDATA, &low_byte);			//read low byte of channel
	Color_Read1Byte(COLOR_GDATAH, &high_byte);		//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : Color_ReadBData
Author 					: Grant Phillips
Date Modified   : 18/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the BLUE color data value from ColorClick Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t Color_ReadBData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	Color_Read1Byte(COLOR_BDATA, &low_byte);			//read low byte of channel
	Color_Read1Byte(COLOR_BDATAH, &high_byte);		//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : HMC6352_Write1Byte
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write 1 byte of data to the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: byte0		-		8-bit data to write to the compass module

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	TRANSMITTER MODE (Address) timeout
									4 - BYTE TRANSMIT timeout
*********************************************************************************************/
uint8_t Color_Write1Byte(uint8_t byte0)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = COLOR_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, COLOR_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte0);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	  
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_Write2Bytes
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write 2 bytes of data to the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: byte1		-		1st 8-bit data to write to the compass module
									byte0		-		2nd 8-bit data to write to the compass module

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	TRANSMITTER MODE (Address) timeout
									4 - BYTE1 TRANSMIT timeout
									5 - BYTE0 TRANSMIT timeout
*********************************************************************************************/
uint8_t Color_Write2Bytes(uint8_t byte1, uint8_t byte0)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = COLOR_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, COLOR_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte1);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte0);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 5;
	}
	  
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_Read1Byte
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read 1 byte of data from the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: *ptr		-		8-bit pointer to store read value 

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	RECEIVER MODE (Address) timeout
									4 - BYTE RECEIVE timeout
*********************************************************************************************/
uint8_t Color_Read1Byte(uint8_t address, uint8_t *ptr)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = COLOR_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
	/* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, COLOR_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 address data */
	I2C_SendData(I2C1, address);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 5;
	}
     
  /* Send slave Address for read */
  I2C_Send7bitAddress(I2C1, COLOR_ADDRESS, I2C_Direction_Receiver);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master receiver */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 6;
	}
	
	/* Prepare an NACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, DISABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = COLOR_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 7;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	*ptr = I2C_ReceiveData(I2C1);
	
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_Read2Bytes
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read 2 bytes of data from the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: array[]		-		8-bit array to store read value (array[0] for first data byte
																and array[1] for second data byte)

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	RECEIVER MODE (Address) timeout
									4 - BYTE1 RECEIVE timeout
									5 - BYTE0 RECEIVE timeout
*********************************************************************************************/
uint8_t HMC6352_Read2Bytes(uint8_t array[])
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = COLOR_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for read */
  I2C_Send7bitAddress(I2C1, COLOR_ADDRESS, I2C_Direction_Receiver);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master receiver */
	timeout = COLOR_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
	/* Prepare an ACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, ENABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = COLOR_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 4;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	array[1] = I2C_ReceiveData(I2C1);
	
	/* Prepare an NACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, DISABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = COLOR_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 5;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	array[0] = I2C_ReceiveData(I2C1);
	
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}


