/*#############################################################
Driver name	    : DHT11_F4.c
Author 					: Grant Phillips
Date Modified   : 14/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the DHT11  
									Humidity-Temperature Sensor.  DHT11_Humidity
									holds the last humidity result (as a % between
									0 and 100) and DHT11_Temperature hold the last
									temperature result (in degrees celcius).  The 
									DHT11_Read function will read the new values
									from the device, but cannot be called in 
									intervals less than 1 second.
									

Requirements    : * STM32F4-Discovery Board

Functions				: DHT11_Init
									DHT11_Read
													  
##############################################################*/

#include <stdio.h>

// Change the settings below if different data pins and timers
// are required to be used
// =============================================================
/* DHT11 data pin configuration */
#define DHT11_GPIO_PIN						GPIO_Pin_15						//GPIO pin to use as data pin for DHT11
#define DHT11_GPIO_PORT						GPIOE									//GPIO port for above pin
#define DHT11_GPIO_PERIPH_CLOCK		RCC_AHB1Periph_GPIOE	//Peripheral clock for above data pin

/* DHT11 timer configuration for pulse signal calculations */
#define DHT11_TIM									TIM9									//Timer to use for pulse signal calculations
unsigned int DHT11_TIM_NO = 9;													//Timer number - should be same as above one
// =============================================================



#define	DHT11_TIMEOUT_MAX					0x0000ffff						//nominal timeout delay to prevent system
																												//lock-up in the case of hardware failure
//exit code used by DHT11_Read()
#define DHT11_NOTDONEYET					0
#define DHT11_OK									1
#define DHT11_ERROR_TIMEOUT				2
#define DHT11_ERROR_CHECKSUM			3
uint8_t DHT11_ExitCode = DHT11_NOTDONEYET;

//global variables for holding the last Humidity and Temperature results
uint8_t DHT11_Temperature=0;
uint8_t DHT11_Humidity=0;



void DHT11_Init(void);
void DHT11_PinAsInput(void);
void DHT11_PinAsOutput(void);
void DHT11_WritePin(uint8_t value);
uint8_t DHT11_ReadPin(void);
uint8_t DHT11_Read(void);



/*********************************************************************************************
Function name   : DHT11_Init
Author 					: Grant Phillips
Date Modified   : 14/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the DHT11 Humidity-Temperature Sensor.

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void DHT11_Init(void)
{
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;	
	uint16_t PrescalerValue = 0;
	
	// ========== Initialize PE15 ==========
	/* Enable GPIOE clock */
  RCC_AHB1PeriphClockCmd(DHT11_GPIO_PERIPH_CLOCK, ENABLE);

	/* Set DHT11 data pin as input */
	DHT11_PinAsInput();
						
							 
	// ========== Initialize TIM9 ==========
  /* TIM clock enable - enable the proper periph timer clock according to settings at the top*/
	switch(DHT11_TIM_NO)
	{
		case 1:	{
							RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
							PrescalerValue = 167;			//PSC value; meanse that TIM register will be incremented every 1us (168000000 / (167+1) = 1000000Hz = 1us
						}break;
		case 2:	{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
							PrescalerValue = 83;			//PSC value; meanse that TIM register will be incremented every 1us (84000000 / (83+1) = 1000000Hz = 1us				
						}break;
		case 3:	{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 4:	{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 5:	{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 6:	{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 7:	{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM7, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 8:	{
							RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM8, ENABLE);
							PrescalerValue = 167;				
						}break;
		case 9:	{
							RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM9, ENABLE);
							PrescalerValue = 167;				
						}break;
		case 10:{
							RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM10, ENABLE);
							PrescalerValue = 167;				
						}break;
		case 11:{
							RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM11, ENABLE);
							PrescalerValue = 167;				
						}break;
		case 12:{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM12, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 13:{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM13, ENABLE);
							PrescalerValue = 83;				
						}break;
		case 14:{
							RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);
							PrescalerValue = 83;				
						}break;
		default:{	
						}break;
	}
  
	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;  									
  TIM_TimeBaseStructure.TIM_Period = 65535 - 1;                 //ARR value value set to maximum - no real use, as we wont be using it as an interrupt
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(DHT11_TIM, &TIM_TimeBaseStructure);

  /* TIM enable counter */
  TIM_Cmd(DHT11_TIM, ENABLE);
}



/*********************************************************************************************
Function name   : DHT11_PinAsInput
Author 					: Grant Phillips
Date Modified   : 15/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Sets the DHT11 data pin as INPUT

Special Note(s) : NONE

Parameters			: NONE

Return value		: NONE
*********************************************************************************************/
void DHT11_PinAsInput(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

	/* Configure DHT11 data pin as input floating */
	GPIO_InitStructure.GPIO_Pin = DHT11_GPIO_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(DHT11_GPIO_PORT, &GPIO_InitStructure);
}



/*********************************************************************************************
Function name   : DHT11_PinAsOutput
Author 					: Grant Phillips
Date Modified   : 15/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Sets the DHT11 data pin as OUTPUT

Special Note(s) : NONE

Parameters			: NONE

Return value		: NONE
*********************************************************************************************/
void DHT11_PinAsOutput(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

	/* Configure DHT11 data pin as output push-pull */
	GPIO_InitStructure.GPIO_Pin = DHT11_GPIO_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(DHT11_GPIO_PORT, &GPIO_InitStructure);
}



/*********************************************************************************************
Function name   : DHT11_WritePin
Author 					: Grant Phillips
Date Modified   : 15/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Sets or clears the DHT11 data pin when in OUTPUT mode

Special Note(s) : NONE

Parameters			: value		-	1 (HIGH) or 0 (LOW)

Return value		: NONE
*********************************************************************************************/
void DHT11_WritePin(uint8_t value)
{
	if(value == 1)
		GPIO_WriteBit(DHT11_GPIO_PORT, DHT11_GPIO_PIN, Bit_SET);		//pin HIGH
	else
		GPIO_WriteBit(DHT11_GPIO_PORT, DHT11_GPIO_PIN, Bit_RESET);	//pin LOW
}



/*********************************************************************************************
Function name   : DHT11_ReadPin
Author 					: Grant Phillips
Date Modified   : 15/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read the DHT11 data pin when in INPUT mode

Special Note(s) : NONE

Parameters			: NONE

Return value		: 1 (HIGH) or 0 (LOW)
*********************************************************************************************/
uint8_t DHT11_ReadPin(void)
{
	return GPIO_ReadInputDataBit(DHT11_GPIO_PORT, DHT11_GPIO_PIN);
}



/*********************************************************************************************
Function name   : DHT11_Read
Author 					: Grant Phillips
Date Modified   : 15/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the humidity and temperature values from the DHT11 

Special Note(s) : NONE

Parameters			: NONE

Return value		: 8-bit exit code	which can be:
									1	-	DHT11_OK
									2 - DHT11_ERROR_TIMEOUT
									3 - DHT11_ERROR_CHECKSUM
*********************************************************************************************/
uint8_t DHT11_Read(void)
{
	uint8_t bits[5];											//buffer array
	uint8_t cnt = 7;											//bit counter
	uint8_t idx = 0;											//array index counter
	
	uint8_t i;
	uint32_t loopCnt;
	uint32_t t;
	uint8_t sum;
	
	/* empty the buffer */
	for(i=0; i<5; i++) 
		bits[i] = 0;
	
	/* send START signal */
	DHT11_PinAsOutput();
	DHT11_WritePin(0);
	TIM_SetCounter(DHT11_TIM, 0);								//clear timer
	while(TIM_GetCounter(DHT11_TIM) < 18000);		//wait 18ms
	
	DHT11_WritePin(1);
	TIM_SetCounter(DHT11_TIM, 0);								//clear timer
	while(TIM_GetCounter(DHT11_TIM) < 40);			//wait 40us
	DHT11_PinAsInput();
	
	/* wait for ACKNOWLEDGE or give TIMEOUT error */
	loopCnt = DHT11_TIMEOUT_MAX;
	while(DHT11_ReadPin() == 0)									//wait for HIGH
	{
		if (loopCnt-- == 0) 
			{
				DHT11_ExitCode = DHT11_ERROR_TIMEOUT;
				return DHT11_ERROR_TIMEOUT;
			}
	}
	loopCnt = DHT11_TIMEOUT_MAX;
	while(DHT11_ReadPin() == 1)									//wait for LOW
	{
		if (loopCnt-- == 0) 
			{
				DHT11_ExitCode = DHT11_ERROR_TIMEOUT;
				return DHT11_ERROR_TIMEOUT;
			}
	}
	
	/* read output from DHT11 - 40 bits => 5 bytes or give TIMEOUT error */
	for(i=0; i<40; i++)
	{
		loopCnt = DHT11_TIMEOUT_MAX;
		while(DHT11_ReadPin() == 0)									//wait for HIGH
		{
			if (loopCnt-- == 0) 
			{
				DHT11_ExitCode = DHT11_ERROR_TIMEOUT;
				return DHT11_ERROR_TIMEOUT;
			}
		}

		t = TIM_GetCounter(DHT11_TIM);							//read timer value

		loopCnt = DHT11_TIMEOUT_MAX;
		while(DHT11_ReadPin() == 1)									//wait for LOW
		{
			if (loopCnt-- == 0) 
			{
				DHT11_ExitCode = DHT11_ERROR_TIMEOUT;
				return DHT11_ERROR_TIMEOUT;
			}
		}

		if ((TIM_GetCounter(DHT11_TIM) - t) > 40) 
			bits[idx] |= (1 << cnt);
		
		if (cnt == 0)   	// next byte?
		{
			cnt = 7;    		// restart at MSB
			idx++;      		// next byte!
		}
		else cnt--;
	}
	
	/* write to the variables */
  // as bits[1] and bits[3] are allways zero they are omitted in formulas.
	DHT11_Humidity = bits[0]; 
	DHT11_Temperature = bits[2]; 

	/* calculate checksum and give CHECKSUM error if incorrect */
	sum = bits[0] + bits[2];  

	if (bits[4] != sum)
	{		
		DHT11_ExitCode = DHT11_ERROR_CHECKSUM;
		return DHT11_ERROR_CHECKSUM;
	}
	
	DHT11_ExitCode = DHT11_OK;
	return DHT11_OK;
}

