/*#############################################################
Driver name	    : HMC6352_F4.c
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the I2C HMC6352 
									Compass Module.

Requirements    : * STM32F4-Discovery Board

Functions				: HMC6352_Init
									HMC6352_Write1Byte
									HMC6352_Write2Bytes
									HMC6352_Read1Byte
									HMC6352_Read2Bytes			
									HMC6352_ReadHeading	
													  
##############################################################*/

#include <stdio.h>

#define	HMC6352_TIMEOUT_MAX		0x0000ffff
#define HMC6352_ADDRESS				0x43

void HMC6352_Init(void);
uint8_t HMC6352_Write1Byte(uint8_t byte0);
uint8_t HMC6352_Write2Bytes(uint8_t byte1, uint8_t byte0);
uint8_t HMC6352_Read1Byte(uint8_t *ptr);
uint8_t HMC6352_Read2Bytes(uint8_t array[]);
int HMC6352_ReadHeading(void);

/*********************************************************************************************
Function name   : HMC6352_Init
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void HMC6352_Init(void)
{
	GPIO_InitTypeDef 	GPIO_InitStructure;	
	I2C_InitTypeDef  	I2C_InitStructure;
	
	//Enable the I2C periph
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
	
	//enable the peripheral clock for the pins used by PB6 for I2C SCL and PB9 for I2C1_SDL
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

	//This sequence sets up the I2C1SDA and I2C1SCL pins so they work correctly with the I2C1 peripheral
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_9; 	// Pins 6(I2C1_SCL) and 9(I2C1_SDA)
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF; 						
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;				
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;					
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;				
	GPIO_Init(GPIOB, &GPIO_InitStructure);								 
	
	//The I2C1_SCL and I2C1_SDA pins are now connected to their AF so that the I2C1 can take over control of the pins
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_I2C1); 
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_I2C1);

	//Enable the I2C peripheral
	I2C_Cmd(I2C1, ENABLE);

	//Set the I2C structure parameters 
	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_OwnAddress1 = 0x00;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = 3000;

	//Initialize the I2C peripheral with selected parameters 
	I2C_Init(I2C1,&I2C_InitStructure);
}



/*********************************************************************************************
Function name   : HMC6352_Write1Byte
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write 1 byte of data to the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: byte0		-		8-bit data to write to the compass module

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	TRANSMITTER MODE (Address) timeout
									4 - BYTE TRANSMIT timeout
*********************************************************************************************/
uint8_t HMC6352_Write1Byte(uint8_t byte0)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = HMC6352_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, HMC6352_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte0);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	  
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_Write2Bytes
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write 2 bytes of data to the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: byte1		-		1st 8-bit data to write to the compass module
									byte0		-		2nd 8-bit data to write to the compass module

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	TRANSMITTER MODE (Address) timeout
									4 - BYTE1 TRANSMIT timeout
									5 - BYTE0 TRANSMIT timeout
*********************************************************************************************/
uint8_t HMC6352_Write2Bytes(uint8_t byte1, uint8_t byte0)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = HMC6352_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, HMC6352_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte1);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte0);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 5;
	}
	  
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_Read1Byte
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read 1 byte of data from the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: *ptr		-		8-bit pointer to store read value 

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	RECEIVER MODE (Address) timeout
									4 - BYTE RECEIVE timeout
*********************************************************************************************/
uint8_t HMC6352_Read1Byte(uint8_t *ptr)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = HMC6352_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for read */
  I2C_Send7bitAddress(I2C1, HMC6352_ADDRESS, I2C_Direction_Receiver);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master receiver */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
	/* Prepare an NACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, DISABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = HMC6352_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 4;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	*ptr = I2C_ReceiveData(I2C1);
	
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_Read2Bytes
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read 2 bytes of data from the HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: array[]		-		8-bit array to store read value (array[0] for first data byte
																and array[1] for second data byte)

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	RECEIVER MODE (Address) timeout
									4 - BYTE1 RECEIVE timeout
									5 - BYTE0 RECEIVE timeout
*********************************************************************************************/
uint8_t HMC6352_Read2Bytes(uint8_t array[])
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = HMC6352_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for read */
  I2C_Send7bitAddress(I2C1, HMC6352_ADDRESS, I2C_Direction_Receiver);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master receiver */
	timeout = HMC6352_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
	/* Prepare an ACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, ENABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = HMC6352_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 4;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	array[1] = I2C_ReceiveData(I2C1);
	
	/* Prepare an NACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, DISABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = HMC6352_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 5;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	array[0] = I2C_ReceiveData(I2C1);
	
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : HMC6352_ReadHeading
Author 					: Grant Phillips
Date Modified   : 13/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the heading value from HMC6352 Compass Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: The heading value (0 - 3600); -1 for any error reading the value
*********************************************************************************************/
int HMC6352_ReadHeading(void)
{
	uint8_t rcv;
	uint8_t buf[2];
	int heading;
	
	rcv = HMC6352_Write1Byte('A');		//Get Data. Compensate and Calculate New Heading
	if(rcv != 0)
		return -1;											//return a error if not succesfull
			
	Delay(10);												//allow 10ms for measurement (datasheet says 6ms)

	rcv = HMC6352_Read2Bytes(buf);		//Read the resultant 2 bytes for the heading value
	if(rcv != 0)
		return -1;											//return a error if not succesfull
	
	heading = (buf[1] << 8) + buf[0];	//make up a integer value with the 2 byte datas
	
	return heading;
}
