/*#############################################################
Driver name	    : ISL29125_F4.c
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the ISL29125 
									Color Sensor Module.

Requirements    : * STM32F4-Discovery Board

Functions				: ISL29125_Init
									ISL29125_Reset
									ISL29125_Config
									ISL29125_ReadRData
									ISL29125_ReadGData
									ISL29125_ReadBData
									ISL29125_Write1Byte
									ISL29125_Write2Bytes
									ISL29125_Read1Byte
									ISL29125_Read2Bytes	

													  
##############################################################*/

#include <stdio.h>

#define	ISL29125_TIMEOUT_MAX			0x0000ffff
#define ISL29125_ADDRESS					0x88

// ISL29125 Registers
#define DEVICE_ID 		0x00
#define CONFIG_1 			0x01
#define CONFIG_2 			0x02
#define CONFIG_3 			0x03
#define THRESHOLD_LL 	0x04
#define THRESHOLD_LH 	0x05
#define THRESHOLD_HL 	0x06
#define THRESHOLD_HH 	0x07
#define STATUS 				0x08 
#define GREEN_L 			0x09 
#define GREEN_H 			0x0A
#define RED_L 				0x0B
#define RED_H 				0x0C
#define BLUE_L 				0x0D
#define BLUE_H 				0x0E

// Configuration Settings
#define CFG_DEFAULT 					0x00

// CONFIG1
// Pick a mode, determines what color[s] the sensor samples, if any
#define CFG1_MODE_POWERDOWN 	0x00
#define CFG1_MODE_G 					0x01
#define CFG1_MODE_R 					0x02
#define CFG1_MODE_B 					0x03
#define CFG1_MODE_STANDBY 		0x04
#define CFG1_MODE_RGB 				0x05
#define CFG1_MODE_RG 					0x06
#define CFG1_MODE_GB 					0x07

// Light intensity range
// In a dark environment 375Lux is best, otherwise 10KLux is likely the best option
#define CFG1_375LUX 					0x00
#define CFG1_10KLUX 					0x08

// Change this to 12 bit if you want less accuracy, but faster sensor reads
// At default 16 bit, each sensor sample for a given color is about ~100ms
#define CFG1_16BIT 						0x00
#define CFG1_12BIT 						0x10

// Unless you want the interrupt pin to be an input that triggers sensor sampling, leave this on normal
#define CFG1_ADC_SYNC_NORMAL 	0x00
#define CFG1_ADC_SYNC_TO_INT 	0x20

// CONFIG2
// Selects upper or lower range of IR filtering
#define CFG2_IR_OFFSET_OFF 		0x00
#define CFG2_IR_OFFSET_ON 		0x80

// Sets amount of IR filtering, can use these presets or any value between 0x00 and 0x3F
// Consult datasheet for detailed IR filtering calibration
#define CFG2_IR_ADJUST_LOW 		0x00
#define CFG2_IR_ADJUST_MID 		0x20
#define CFG2_IR_ADJUST_HIGH 	0x3F

// CONFIG3
// No interrupts, or interrupts based on a selected color
#define CFG3_NO_INT 					0x00
#define CFG3_G_INT 						0x01
#define CFG3_R_INT 						0x02
#define CFG3_B_INT 						0x03

// How many times a sensor sample must hit a threshold before triggering an interrupt
// More consecutive samples means more times between interrupts, but less triggers from short transients
#define CFG3_INT_PRST1 				0x00
#define CFG3_INT_PRST2 				0x04
#define CFG3_INT_PRST4 				0x08
#define CFG3_INT_PRST8 				0x0C

// If you would rather have interrupts trigger when a sensor sampling is complete, enable this
// If this is disabled, interrupts are based on comparing sensor data to threshold settings
#define CFG3_RGB_CONV_TO_INT_DISABLE 	0x00
#define CFG3_RGB_CONV_TO_INT_ENABLE 	0x10

// STATUS FLAG MASKS
#define FLAG_INT 							0x01
#define FLAG_CONV_DONE 				0x02
#define FLAG_BROWNOUT 				0x04
#define FLAG_CONV_G 					0x10
#define FLAG_CONV_R 					0x20
#define FLAG_CONV_B 					0x30



uint8_t ISL29125_Write1Byte(uint8_t byte0);
uint8_t ISL29125_Write2Bytes(uint8_t byte1, uint8_t byte0);
uint8_t ISL29125_Read1Byte(uint8_t address, uint8_t *ptr);
uint8_t ISL29125_Read2Bytes(uint8_t array[]);
uint8_t ISL29125_Init(void);
void ISL29125_Reset(void);
void ISL29125_Config(uint8_t config1, uint8_t config2, uint8_t config3);
uint16_t ISL29125_ReadRData(void);
uint16_t ISL29125_ReadGData(void);
uint16_t ISL29125_ReadBData(void);



/*********************************************************************************************
Function name   : ISL29125_Init
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the ISL29125 Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE
Return value		: 0	-	Successfull
									1	-	Error initializing ISL29125 Color Sensor
*********************************************************************************************/
uint8_t ISL29125_Init(void)
{
	uint8_t data=0x00;
	
	GPIO_InitTypeDef 	GPIO_InitStructure;	
	I2C_InitTypeDef  	I2C_InitStructure;
	
	//========== initializethe I2C bus ==========
	
	//Enable the I2C periph
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
	
	//enable the peripheral clock for the pins used by PB6 for I2C SCL and PB9 for I2C1_SDL
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

	//This sequence sets up the I2C1SDA and I2C1SCL pins so they work correctly with the I2C1 peripheral
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_9; 	// Pins 6(I2C1_SCL) and 9(I2C1_SDA)
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF; 						
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;				
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;					
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;				
	GPIO_Init(GPIOB, &GPIO_InitStructure);								 
	
	//The I2C1_SCL and I2C1_SDA pins are now connected to their AF so that the I2C1 can take over control of the pins
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_I2C1); 
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_I2C1);

	//Enable the I2C peripheral
	I2C_Cmd(I2C1, ENABLE);

	//Set the I2C structure parameters 
	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_OwnAddress1 = 0x00;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = 3000;

	//Initialize the I2C peripheral with selected parameters 
	I2C_Init(I2C1,&I2C_InitStructure);
	
	
	//========== initialize the the actual ISL29125 Color Sensor module ==========
	// Check device ID
	ISL29125_Read1Byte(DEVICE_ID, &data);
	if (data != 0x7D) 
		return 1;
	// Reset registers 
	ISL29125_Reset(); 
	// Check reset
	ISL29125_Read1Byte(CONFIG_1, &data);
	if(data != 0x00)
		return 1;
	ISL29125_Read1Byte(CONFIG_2, &data);
	if(data != 0x00)
		return 1;
	ISL29125_Read1Byte(CONFIG_3, &data);
	if(data != 0x00)
		return 1;
	ISL29125_Read1Byte(STATUS, &data);
	if(data != 0x00)
		return 1;
	// Set to RGB mode, 10k lux, and high IR compensation
	ISL29125_Config(CFG1_MODE_RGB | CFG1_10KLUX, CFG2_IR_ADJUST_HIGH, CFG_DEFAULT);
	
	return 0;
}



/*********************************************************************************************
Function name   : ISL29125_Reset
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reset all registers in the ISL29125 Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void ISL29125_Reset(void)
{
	ISL29125_Write2Bytes(DEVICE_ID, 0x46);
}



/*********************************************************************************************
Function name   : ISL29125_Config
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Setup Configuration registers (three registers) for the ISL29125 Color Sensor 
									Module.

Special Note(s) : NONE

Parameters			: config1	-	CONFIG1 variables from #defines at the top of this file
									config2	-	CONFIG2 variables from #defines at the top of this file
									config3	-	CONFIG3 variables from #defines at the top of this file

Return value		: NONE
*********************************************************************************************/
void ISL29125_Config(uint8_t config1, uint8_t config2, uint8_t config3)
{
	ISL29125_Write2Bytes(CONFIG_1, config1);
	ISL29125_Write2Bytes(CONFIG_2, config2);
	ISL29125_Write2Bytes(CONFIG_3, config3);
}



/*********************************************************************************************
Function name   : ISL29125_ReadRData
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the RED color data value from ISL29125 Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t ISL29125_ReadRData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	ISL29125_Read1Byte(RED_L, &low_byte);			//read low byte of channel
	ISL29125_Read1Byte(RED_H, &high_byte);		//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : ISL29125_ReadGData
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the GREEN color data value from ISL29125 Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t ISL29125_ReadGData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	ISL29125_Read1Byte(GREEN_L, &low_byte);			//read low byte of channel
	ISL29125_Read1Byte(GREEN_H, &high_byte);		//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : ISL29125_ReadBData
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the BLUE color data value from ISL29125 Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE

Return value		: 16-bit color value
*********************************************************************************************/
uint16_t ISL29125_ReadBData(void)
{
	uint8_t low_byte, high_byte;
  uint16_t out_color;
	
	ISL29125_Read1Byte(BLUE_L, &low_byte);			//read low byte of channel
	ISL29125_Read1Byte(BLUE_H, &high_byte);			//read high byte of channel
	out_color = (uint16_t)high_byte;
	out_color = (out_color << 8);
  out_color = (out_color | low_byte);
  return out_color;
}



/*********************************************************************************************
Function name   : ISL29125_Write1Byte
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write 1 byte of data to the ISL29125 Color Sensor.

Special Note(s) : NONE

Parameters			: byte0		-		8-bit data to write to the compass module

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	TRANSMITTER MODE (Address) timeout
									4 - BYTE TRANSMIT timeout
*********************************************************************************************/
uint8_t ISL29125_Write1Byte(uint8_t byte0)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = ISL29125_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, ISL29125_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte0);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	  
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : ISL29125_Write2Bytes
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write 2 bytes of data to the ISL29125 Color Sensor.

Special Note(s) : NONE

Parameters			: byte1		-		1st 8-bit data to write to the compass module
									byte0		-		2nd 8-bit data to write to the compass module

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	TRANSMITTER MODE (Address) timeout
									4 - BYTE1 TRANSMIT timeout
									5 - BYTE0 TRANSMIT timeout
*********************************************************************************************/
uint8_t ISL29125_Write2Bytes(uint8_t byte1, uint8_t byte0)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = ISL29125_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, ISL29125_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte1);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	
  /* Send I2C1 byte data */
	I2C_SendData(I2C1, byte0);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 5;
	}
	  
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : ISL29125_Read1Byte
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read 1 byte of data from the ISL29125 Color Sensor.

Special Note(s) : NONE

Parameters			: *ptr		-		8-bit pointer to store read value 

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	RECEIVER MODE (Address) timeout
									4 - BYTE RECEIVE timeout
*********************************************************************************************/
uint8_t ISL29125_Read1Byte(uint8_t address, uint8_t *ptr)
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = ISL29125_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
	/* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for write */
  I2C_Send7bitAddress(I2C1, ISL29125_ADDRESS, I2C_Direction_Transmitter);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master transmitter */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
  /* Send I2C1 address data */
	I2C_SendData(I2C1, address);
     
	/* Test on I2C1 EV8_2 and clear it --> byte has been transmitted */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	{
		if ((timeout--) == 0) return 4;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 5;
	}
     
  /* Send slave Address for read */
  I2C_Send7bitAddress(I2C1, ISL29125_ADDRESS, I2C_Direction_Receiver);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master receiver */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 6;
	}
	
	/* Prepare an NACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, DISABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = ISL29125_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 7;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	*ptr = I2C_ReceiveData(I2C1);
	
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}



/*********************************************************************************************
Function name   : ISL29125_Read2Bytes
Author 					: Grant Phillips
Date Modified   : 07/06/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read 2 bytes of data from the ISL29125 Color Sensor.

Special Note(s) : NONE

Parameters			: array[]		-		8-bit array to store read value (array[0] for first data byte
																and array[1] for second data byte)

Return value		: 0	-	Successfull
									1	-	FLAG BUSY timeout
									2	-	START CONDITION timeout
									3	-	RECEIVER MODE (Address) timeout
									4 - BYTE1 RECEIVE timeout
									5 - BYTE0 RECEIVE timeout
*********************************************************************************************/
uint8_t ISL29125_Read2Bytes(uint8_t array[])
{
	uint32_t timeout;
	
	/* Wait until I2C1 is not busy anymore */
	timeout = ISL29125_TIMEOUT_MAX;
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY))
	{
		if ((timeout--) == 0) return 1;
	}
	
  /* Send I2C1 START condition */
  I2C_GenerateSTART(I2C1, ENABLE);
   
  /* Test on I2C1 EV5 and clear it --> Slave has acknowledged start condition */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT))
	{
		if ((timeout--) == 0) return 2;
	}
     
  /* Send slave Address for read */
  I2C_Send7bitAddress(I2C1, ISL29125_ADDRESS, I2C_Direction_Receiver);
   
  /* Test on I2C1 EV6 and clear it --> Slave has acknowledged Master receiver */
	timeout = ISL29125_TIMEOUT_MAX;
  while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
	{
		if ((timeout--) == 0) return 3;
	}
	
	/* Prepare an ACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, ENABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = ISL29125_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 4;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	array[1] = I2C_ReceiveData(I2C1);
	
	/* Prepare an NACK for the next data received */
	I2C_AcknowledgeConfig(I2C1, DISABLE);  
	
  /* Test on I2C1 EV7 and clear it --> wait until one byte has been received */
	timeout = ISL29125_TIMEOUT_MAX;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED))
	{
		if ((timeout--) == 0) return 5;
	}
     
	/* Read data from I2C data register and store it at the pointer variable */
	array[0] = I2C_ReceiveData(I2C1);
	
  /* Send I2C1 STOP Condition */
  I2C_GenerateSTOP(I2C1, ENABLE);

	return 0;		//no error
}


