/*#############################################################
Driver name	    : LIS3DSH_F4.c
Author 					: Grant Phillips
Date Modified   : 23/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the LIS3DSH
									3D Accelerometer module found on the STM32F4-
									Discovery board.

Requirements    : * STM32F4-Discovery Board

Functions				: LIS3DSH_Init
									LIS3DSH_WriteByte
									LIS3DSH_ReadByte
									LIS3DSH_ReadAngles
													  
Special Note(s) : NONE
##############################################################*/

#include <math.h>

#define LIS3DSH_OFF_X					0x10
#define LIS3DSH_OFF_Y					0x11
#define LIS3DSH_OFF_Z					0x12
#define LIS3DSH_CTRL_REG3    	0x23  
#define LIS3DSH_CTRL_REG4    	0x20  
#define LIS3DSH_CTRL_REG5    	0x24  
#define LIS3DSH_CTRL_REG6    	0x25  
#define LIS3DSH_OUT_TEMP     	0x0C    
#define LIS3DSH_OUT_X_L      	0x28  
#define LIS3DSH_OUT_X_H      	0x29 
#define LIS3DSH_OUT_Y_L      	0x2A  
#define LIS3DSH_OUT_Y_H				0x2B  
#define LIS3DSH_OUT_Z_L				0x2C  
#define LIS3DSH_OUT_Z_H      	0x2D  
#define LIS3DSH_FIFO_CTRL_REG	0x2E  
#define LIS3DSH_WHO_AM_I			0x0F

#define LIS3DSH_CS_LOW()		GPIO_WriteBit(GPIOE, GPIO_Pin_3, Bit_RESET) 			//set LIS3DSH_CS low
#define LIS3DSH_CS_HIGH()		GPIO_WriteBit(GPIOE, GPIO_Pin_3, Bit_SET) 				//set LIS3DSH_CS high
	


uint8_t LIS3DSH_SPI1_send(uint8_t data);
uint8_t LIS3DSH_Init(void);
void LIS3DSH_Init2(void);
uint8_t LIS3DSH_ReadByte(uint8_t address);
void LIS3DSH_WriteByte(uint8_t address, uint8_t data);
float gToDegrees(float V, float H);
void LIS3DSH_ReadAngles(float *pRoll, float *pPitch);



uint8_t LIS3DSH_SPI1_send(uint8_t data)
{
	SPI_I2S_SendData(SPI1, data);																	//write data to be transmitted to the SPI
	while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE));				//wait until transmit complete	
	while(!SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE));				//wait until receive complete
	return(SPI_I2S_ReceiveData(SPI1));														//return received data from SPI data register	
}



/*********************************************************************************************
Function name   : LIS3DSH_Init
Author 					: Grant Phillips
Date Modified   : 23/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the LIS3DSH module

Special Note(s) : NONE

Parameters			: NONE
Return value		: 0	-	LIS3DSH module initialized correctly
									1	-	module did not initialize correctly and is faulty or missing
*********************************************************************************************/
uint8_t LIS3DSH_Init(void)
{
	GPIO_InitTypeDef 	GPIO_InitStructure;	
	SPI_InitTypeDef 	SPI_InitStruct;
	uint8_t ReadByte;
	
	/* GPIO Peripheral Clocks */
	RCC_AHB1PeriphClockCmd (RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOE, ENABLE);

	/* Configure GPIO pins to SPI alternate function */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);
	
	/* Configure the chip select pin PE2 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
	
	GPIO_WriteBit(GPIOE, GPIO_Pin_3, Bit_SET); 	// set LIS3DSH_CS high
	
	/* Enable SPI2 peripheral clock */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
	
	/* configure SPI1 in Mode 0 */
	SPI_InitStruct.SPI_Direction = SPI_Direction_2Lines_FullDuplex; 	// set to full duplex mode, seperate MOSI and MISO lines
	SPI_InitStruct.SPI_Mode = SPI_Mode_Master;     										// transmit in master mode, NSS pin has to be always high
	SPI_InitStruct.SPI_DataSize = SPI_DataSize_8b; 										// one packet of data is 8 bits wide
	SPI_InitStruct.SPI_CPOL = SPI_CPOL_Low;        										// clock is low when idle
	SPI_InitStruct.SPI_CPHA = SPI_CPHA_1Edge;      										// data sampled at first edge
	SPI_InitStruct.SPI_NSS = SPI_NSS_Soft| SPI_NSSInternalSoft_Set; 	// set the NSS management to internal and pull internal NSS high
	SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256; // SPI frequency is APB2 frequency / 256
	SPI_InitStruct.SPI_FirstBit = SPI_FirstBit_MSB;										// data is transmitted MSB first
	SPI_Init(SPI1, &SPI_InitStruct); 
	
	/* Enable SPI1 */
	SPI_Cmd(SPI1, ENABLE);
	
	/* Check if LIS3DSH responds with correct ID */
	ReadByte = LIS3DSH_ReadByte(LIS3DSH_WHO_AM_I);
	if(ReadByte != 0x3F)
		return 1;																				//ID not detected; LIS3DSH faulty or missing

	/* continue LIS3DSH setup */

	
	//LIS3DSH_WriteByte(LIS3DSH_CTRL_REG3, 0x01);   	// resetting the accelerometer internal circuit
	//LIS3DSH_WriteByte(LIS3DSH_CTRL_REG4, 0x67);   	// 100Hz data update rate, block data update disable, x/y/z enabled 
  //LIS3DSH_WriteByte(LIS3DSH_CTRL_REG5, 0x20);   	// Anti aliasing filter bandwidth 800Hz, 16G (very sensitive), no self-test, 4-wire interface
	//LIS3DSH_WriteByte(LIS3DSH_FIFO_CTRL_REG, 0x00);	// configure FIFO for bypass mode
  
	//LIS3DSH_WriteByte(0x10, 0x00);           				// Output(X) = Measurement(X) - OFFSET(X) * 32;
  //LIS3DSH_WriteByte(0x11, 0x00);           				// Output(Y) = Measurement(Y) - OFFSET(Y) * 32;
  //LIS3DSH_WriteByte(0x12, 0x00);          				// Output(Z) = Measurement(Z) - OFFSET(Z) * 32;
	
	LIS3DSH_WriteByte(LIS3DSH_CTRL_REG4, 0x5F);  			// Normal power mode, all axes enabled, 50 Hz ODR
	LIS3DSH_WriteByte(LIS3DSH_CTRL_REG5, 0x80); 			// 200 Hz antialias filter, +/- 2g FS range   
	LIS3DSH_WriteByte(LIS3DSH_FIFO_CTRL_REG, 0);			// configure FIFO for bypass mode   
	LIS3DSH_WriteByte(LIS3DSH_CTRL_REG6, 0x10);				// disable FIFO, enable register address auto-increment  
	
	/* these two lines prevents lock-up of sampling according to:
	https://my.st.com/public/STe2ecommunities/mems_sensors/Lists/Accelerometers/DispForm.aspx?ID=304&Source=/public/STe2ecommunities/mems_sensors/Tags.aspx?tags=sampling
	
	Not sure why it works
	*/
	LIS3DSH_WriteByte(LIS3DSH_CTRL_REG4, 0x00);
	LIS3DSH_WriteByte(LIS3DSH_CTRL_REG4, 0x37);
	
	return 0;
}



/*********************************************************************************************
Function name   : LIS3DSH_ReadByte
Author 					: Grant Phillips
Date Modified   : 23/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Read data from the LIS3DSH module from a selected address

Special Note(s) : NONE

Parameters			: address	-	8-bit address from where the data should be read
Return value		: 8-bit data byte 
*********************************************************************************************/
uint8_t LIS3DSH_ReadByte(uint8_t address)
{
	uint8_t data;
	
	LIS3DSH_CS_LOW();			
	
	LIS3DSH_SPI1_send(0x80 | address);		//READ instruction
	data = LIS3DSH_SPI1_send(0xff);				//use 0x00 as dummy data to generate 8 clock pulses for reading data from MISO				
	
	LIS3DSH_CS_HIGH();
	return(data);
}



/*********************************************************************************************
Function name   : LIS3DSH_WriteByte
Author 					: Grant Phillips
Date Modified   : 23/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Write a byte of data to the LIS3DSH module at a selected address

Special Note(s) : NONE

Parameters			: address	-	8-bit address to where the data should be written
									data		-	8-bit data byte to write
Return value		: NONE
*********************************************************************************************/
void LIS3DSH_WriteByte(uint8_t address, uint8_t data)
{
	LIS3DSH_CS_LOW();
	
	LIS3DSH_SPI1_send(address);											
	LIS3DSH_SPI1_send(data);																	
	
	LIS3DSH_CS_HIGH();
}



/*********************************************************************************************
Function name   : gToDegrees
Author 					: Sergey Ostrikov VK
Date Modified   : 23/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Converts velocity in vectors into a angle

Special Note(s) : NONE

Parameters			: V	-	?
									H	-	?
Return value		: angle
*********************************************************************************************/
float gToDegrees(float V, float H)               // refer to the orientation pic above
{
  float retval;
  uint16_t orientation=0;
 
  if (H == 0) H = 0.001;                         // preventing division by zero
  if (V == 0) V = 0.001;                         // preventing division by zero
 
  if ((H > 0) && (V > 0)) orientation = 0;
  if ((H < 0) && (V > 0)) orientation = 90; 
  if ((H < 0) && (V < 0)) orientation = 180;
  if ((H > 0) && (V < 0)) orientation = 270;
 
  retval = ((atan(V/H)/3.14159)*180);
  if (retval < 0) retval += 90;
  retval = fabs(retval) + orientation;
  return retval;
}



/*********************************************************************************************
Function name   : LIS3DSH_ReadAngles
Author 					: Grant Phillips (modified from Sergey Ostrikov VK)
Date Modified   : 23/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the roll angle and pitch angle from the LIS3DSH module.

Special Note(s) : All the calculations in this function was done by ST Microelectronics

Parameters			: *pRoll			-	reference to variable for the roll angle (0.0 - 89.999999)
									*pPitch			-	reference to variable for the pitch angle (0.0 - 89.999999)
Return value		: NONE
*********************************************************************************************/
void LIS3DSH_ReadAngles(float *pRoll, float *pPitch)
{
	uint8_t xLSB=0, xMSB, yLSB, yMSB, zLSB, zMSB; 			// 8-bit values from accelerometer
	int16_t Xg, Yg, Zg;                                 // 16-bit values from accelerometer
	
	xMSB = LIS3DSH_ReadByte(LIS3DSH_OUT_X_H);						// read X high byte register
	xLSB = LIS3DSH_ReadByte(LIS3DSH_OUT_X_L);						// read X low byte register
	yMSB = LIS3DSH_ReadByte(LIS3DSH_OUT_Y_H);
	yLSB = LIS3DSH_ReadByte(LIS3DSH_OUT_Y_L);
	zMSB = LIS3DSH_ReadByte(LIS3DSH_OUT_Z_H);
	zLSB = LIS3DSH_ReadByte(LIS3DSH_OUT_Z_L);
		
	//pack MSB and LSB bytes for X, Y, and Z
	Xg = (xMSB << 8) | (xLSB);   
	Yg = (yMSB << 8) | (yLSB);
	Zg = (zMSB << 8) | (zLSB);		

	//Convert X, Y, and Z accelerometer values to meters per second squared
	Xg /= -141;                          
	Zg /= -141;                          
	Yg /= -141;
		
	*pRoll = gToDegrees(Zg, Xg);                				// get degrees between Z and X planes
	*pPitch = gToDegrees(Zg, Yg);												// get degrees between Z and Y planes
}
