/*#############################################################
Driver name	    : TCS3200_F4.c
Author 					: Grant Phillips
Date Modified   : 31/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the TCS3200 
									Color Sensor Module.

Requirements    : * STM32F4-Discovery Board

Functions				: TCS3200_Init
									TCS3200_SetLEDs
									TCS3200_SetMode
									TCS3200_ReadPulse
													  
##############################################################*/

#include <stdio.h>

// ==================== MACROS ====================
/* Pins configuration */
#define S0_LOW				GPIO_WriteBit(GPIOD, GPIO_Pin_8, Bit_RESET)
#define S0_HIGH				GPIO_WriteBit(GPIOD, GPIO_Pin_8, Bit_SET)
#define S1_LOW				GPIO_WriteBit(GPIOD, GPIO_Pin_9, Bit_RESET)
#define S1_HIGH				GPIO_WriteBit(GPIOD, GPIO_Pin_9, Bit_SET)
#define S2_LOW				GPIO_WriteBit(GPIOD, GPIO_Pin_10, Bit_RESET)
#define S2_HIGH				GPIO_WriteBit(GPIOD, GPIO_Pin_10, Bit_SET)
#define S3_LOW				GPIO_WriteBit(GPIOD, GPIO_Pin_11, Bit_RESET)
#define S3_HIGH				GPIO_WriteBit(GPIOD, GPIO_Pin_11, Bit_SET)
#define LEDS_LOW			GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_RESET)
#define LEDS_HIGH			GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_SET)



// ==================== Instruction Mnemonics ====================
#define TCS3200_OFF						0
#define TCS3200_SCALE_2				1
#define TCS3200_SCALE_20			2
#define TCS3200_SCALE_100			3

#define TCS3200_RED						0
#define TCS3200_GREEN					1
#define TCS3200_BLUE					2
#define TCS3200_CLEAR					3



// ==================== FUNCTION PROTOTYPES ====================
void TCS3200_Init(void);
void TCS3200_SetLEDs(uint8_t OnOff);
void TCS3200_SetMode(uint8_t mode);
uint32_t TCS3200_ReadPulse(uint8_t color);


/*********************************************************************************************
Function name   : TCS3200_Init
Author 					: Grant Phillips
Date Modified   : 31/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the TCS3200 Color Sensor Module.

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void TCS3200_Init(void)
{
	GPIO_InitTypeDef 	GPIO_InitStructure;	
	TIM_ICInitTypeDef TIM_ICInitStructure;									//structure for TIM Input Compare
	
	//========== initialize the GPIO pins for S0, S1, S2, S3, and LEDs output ==========
	
	/* Enable GPIOD clock */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	
	/* Configure PD7 - 11 as output push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_10 | GPIO_Pin_9 | GPIO_Pin_8 | GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	
	//========== initialize TIM5 ==========
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);			//enable the TIM5 clock
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);			//enable the AHB1 Peripheral Clock to use GPIOA
  
	/* Pin configuration */
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_1;								//configure PA1 in alternate function
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  //GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_TIM5);		//connect the pin to the desired peripherals' Alternate Function (AF) - TIM5
	
	/* Input Control (IC) configuration - PWM1 Mode configuration: Channel1 */
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;					//configre TIM5 for input-PWM mode
  TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_PWMIConfig(TIM5, &TIM_ICInitStructure);

	TIM_SelectInputTrigger(TIM5, TIM_TS_TI2FP2);
  TIM_SelectSlaveMode(TIM5, TIM_SlaveMode_Reset);
  TIM_SelectMasterSlaveMode(TIM5,TIM_MasterSlaveMode_Enable);

  TIM_Cmd(TIM5, ENABLE);																		//enable TIM5
	
	
	//========== initializethe the actual TCS3200 module ==========

	TCS3200_SetMode(3);			//100% scaling
}



/*********************************************************************************************
Function name   : TCS3200_SetLEDs
Author 					: Grant Phillips
Date Modified   : 31/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Controls the output of the White LEDs of the ST3200 Color Sensor Module.  

Special Note(s) : NONE

Parameters			: OnOff (1=ON; 0=OFF)

Return value		: NONE
*********************************************************************************************/
void TCS3200_SetLEDs(uint8_t OnOff)
{
	if(OnOff == 1)
		LEDS_HIGH;
	else if(OnOff == 0)
		LEDS_LOW;
	else
		LEDS_LOW;
}



/*********************************************************************************************
Function name   : TCS3200_SetMode
Author 					: Grant Phillips
Date Modified   : 31/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Sets the mode of operation for the ST3200 Color Sensor Module.  

Special Note(s) : NONE

Parameters			: mode (TCS3200_OFF, TCS3200_SCALE_2, TCS3200_SCALE_20 or TCS3200_SCALE_100)

Return value		: NONE
*********************************************************************************************/
void TCS3200_SetMode(uint8_t mode)
{
	if(mode == 0)				//LEDs Off and TCS3200 in power down
	{
		LEDS_LOW;					
		S0_LOW;						
		S1_LOW;
	}
	else if(mode == 1)	//Output frequency at 2% scaling
	{
		S0_LOW;						
		S1_HIGH;
	}
	else if(mode == 2)	//Output frequency at 20% scaling
	{
		S0_HIGH;						
		S1_LOW;
	}
	else if(mode == 3)	//Output frequency at 100% scaling
	{
		S0_HIGH;						
		S1_HIGH;
	}
	else
	{
		LEDS_LOW;					
		S0_LOW;						
		S1_LOW;
	}
}



/*********************************************************************************************
Function name   : TCS3200_ReadPulse
Author 					: Grant Phillips
Date Modified   : 31/08/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Reads the output frequency's HIGH pulse from ST3200 Color Sensor Module for 
									a specific color.

Special Note(s) : NONE

Parameters			: color (TCS3200_RED, TCS3200_GREEN, TCS3200_BLUE or TCS3200_CLEAR)

Return value		: 32-bit unsigned integer representing the high pulse in nS (nano seconds)
*********************************************************************************************/
uint32_t TCS3200_ReadPulse(uint8_t color)
{
	//Set S3 and S4 according to the required color
	if(color == TCS3200_RED)
	{	
		S2_LOW;						
		S3_LOW;
	}
	else if(color == TCS3200_GREEN)
	{	
		S2_HIGH;						
		S3_HIGH;
	}
	else if(color == TCS3200_BLUE)
	{	
		S2_LOW;						
		S3_HIGH;
	}
	else if(color == TCS3200_CLEAR)
	{	
		S2_HIGH;						
		S3_LOW;
	}
	else				//default is RED
	{			
		S0_LOW;						
		S1_LOW;
	}
	
	//Delay to allow frequency to change for the set color
	Delay(1);
	
	return((TIM_GetCapture1(TIM5) * 1000 /84));
}


