/*#############################################################
File Name       : ex1_HC-SR04_measure_swv.c
Author          : Grant Phillips
Date Modified   : 03/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Example program that demonstrates the use of
                  an HC-SR04 ultrasonic distance sensor.
									
									The output is displayed on a PC using the
									Serial Wire Viewer (SWV).  See the SWV examples
									for an explanation of the use of this feature.

Requirements    : * STM32F4-Discovery Board
              
Circuit         : * A wire link between PB3 and pin6 of the SWD
										connector (CN3) as explained in the SWV
										examples section
									* The HC-SR04 ultrasonic sensor connected as:
										VCC		-		+5V
										TRIG	-		PB8
										ECHO	-		PA1
										GND		-		GND
										
See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void HC_SR04_Init(void);																				//function prototype for a user funtion HC_SR04_Init()

/* #global variables -----------------------------------------*/



int main(void)
{ RCC_ClocksTypeDef 			RCC_Clocks;									//structure used for setting up the SysTick Interrupt
	
  uint32_t distance;																	//declase a 32-bit unsigned int to hold the distance value
	uint32_t mm;																				//32-bit unsigned int to hold the distance value in mm
	unsigned int dec, frac;															//used for converting the integer distance to a float value for printing
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																					//will end up in this infinite loop if there was an error with Systick_Config
	
	HC_SR04_Init();																			//initialize the HC-SR04 ultrasonic sensor - see the function definition below
		
	/* Main program loop */
  while (1)
  {
		distance = (TIM_GetCapture1(TIM5)/84);						//read the TIM5 value and divide by 84 (84Mhz - TIM5 clock rate); the result is in us.
		
		printf("%d us        " , distance);   					    

		//HC-SR04 formula: distance (in m) = pulse in seconds * 340 / 2
		mm = distance * 340 / 2;													//mm is in micro meters (because distance is in us), e.g. if distance = 728us, then mm = 123760um
		
		dec = mm / 1000;																	//calculate the integer part, e.g. 123
		frac = mm - (dec * 1000);													//calculate the fraction part, e.g. 760
		printf("%d.%03d mm\n" , dec, frac);   						//%03d prints a integer value in a field of 3 characters with leading 0's
		
    Delay(10);
  }	
}



/*
Retarget the C library printf function to the SWO Viewer.
Overwrites int fputc(int ch, FILE *f) function used by printf.
*/								
int fputc(int ch, FILE *f)
{
	return(ITM_SendChar(ch));
}



void HC_SR04_Init(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;								//structure for GPIO setup
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;						//structure for TIM Time Base
	TIM_OCInitTypeDef				TIM_OCInitStructure;							//structure for TIM Output Compare
	TIM_ICInitTypeDef  			TIM_ICInitStructure;							//structure for TIM Input Compare
	
	/* ===== Configure the TRIG pin =====
	
	PB8 will be used as the TRIGGER pin in this example.  It will be used in PWM mode and will generate
	a 10us HIGH pulse every 60ms to trigger the HC-SR04 to do a measurement.  PB8 is shared TIM4_CH3
	(Timer 4 Channel 3) and also TIM10_CH1 (see the STM32F4-Discovery User Manual p23).  We will use
	TIM10 in this example, seeing that the SERVO_F4.c driver make use of TIM4 for most of the servo outputs.
	Also see Chpt 16 on General Purpose Timers (TIM9 to TIM14) in the STM32F40xxx Reference Manual (RM0090) for
	more detail on the use of TIM10.
	
	Please note that any other free pin that shares a TIM could be used on the STM32F4-Discovery board.  Just
	make sure that the pin is not used by any of the onboard peripherals (p20-29 in the User Manual).	
	*/
	

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);			//enable the AHB Peripheral Clock to use GPIOB
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM10, ENABLE);			//enable the TIM10 clock
	
	/* Pin configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;									//configure PB8 in alternate function pushpull mode
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;						
  GPIO_Init(GPIOB, &GPIO_InitStructure);											
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_TIM10);	//connect the pin to the desired peripherals' Alternate Function (AF) - TIM10
	
	/* Time base configuration */ 
	//The MCU on the STM32F4-Discovery board is clocked at 168Mhz and APB2 also at 168Mhz and we need a 16.667Hz timebase (60ms period)
	//Update_event = TIM_CLK  /((PSC + 1)*(ARR   + 1)*(RCR + 1))
	//			16.667 = 168000000/((167 + 1)*(59999 + 1)*(0   + 1))
  TIM_TimeBaseStructure.TIM_Prescaler = 167;  							//PSC value; meanse that TIM register will be incremented every 1us (168000000 / (167+1) = 1000000Hz = 1us
  TIM_TimeBaseStructure.TIM_Period = 59999;                 //ARR value; create a time period of 60ms (1us * (59999 + 1) = 60ms)
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM10, &TIM_TimeBaseStructure);

  /* PWM1 Mode configuration: Channel1 */
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;				//10% duty cycle 
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  TIM_OC1Init(TIM10, &TIM_OCInitStructure);
  TIM_OC1PreloadConfig(TIM10, TIM_OCPreload_Enable);

  TIM_ARRPreloadConfig(TIM10, ENABLE);

  /* TIM3 enable counter */
  TIM_Cmd(TIM10, ENABLE);

	/*set the pulse width to 10us (10 * 1us = 10us as described in the the timebase section) */
	TIM_SetCompare1(TIM10, 10);																//set 10us HIGH pulse

	/* ===== Configure the ECHO pin =====
	PA1 will be used as the ECHO pin in this example.  It will be used in PWM INPUT mode and will use a counter
	tied to a timer to determine the pulse width of the signal from the HC-SR04 which represent the distance.
	PA1 is shared with TIM2_CH2 and TIM5_CH2 (see the STM32F4-Discovery User Manual p20).  We will use TIM5 in 
	this example, seeing that the SERVO_F4.c driver make use of TIM2 for most of the servo outputs.  
	Also see Chpt 15 on General Purpose Timers (TIM2/3/4) in the STM32F40xxx Reference Manual (RM0090) for
	more detail on the use of TIM5 as PWM INPUT mode.  The TIM5 register value will be updated every
	60ms with a new value for the resultant distance.  A read from the TIM5 register will thus provide the latest
	distance measured.  Not software code is required to trigger the sensor and to measure the pulse, it is all
	done by using a PWM output (TRIG) and a PWM input (ECHO) and using the associated hardware peripherals.
	
	Please note that any other free pin that shares a TIM could be used on the STM32F4-Discovery board.  Just
	make sure that the pin is not used by any of the onboard peripherals (p20-29 in the User Manual).
	*/
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);			//enable the TIM5 clock
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);			//enable the AHB1 Peripheral Clock to use GPIOA
  
	/* Pin configuration */
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_1;								//configure PA1 in alternate function pushpull mode
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_TIM5);		//connect the pin to the desired peripherals' Alternate Function (AF) - TIM5
	
	/* Input Control (IC) configuration - PWM1 Mode configuration: Channel1 */
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;					//configre TIM5 for input-PWM mode
  TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_PWMIConfig(TIM5, &TIM_ICInitStructure);

	TIM_SelectInputTrigger(TIM5, TIM_TS_TI2FP2);
  TIM_SelectSlaveMode(TIM5, TIM_SlaveMode_Reset);
  TIM_SelectMasterSlaveMode(TIM5,TIM_MasterSlaveMode_Enable);

  TIM_Cmd(TIM5, ENABLE);																		//enable TIM5
}








