/*#############################################################
File Name       : ex1_flowsensor_swv.c
Author          : Grant Phillips
Date Modified   : 09/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Example program that demonstrates the use of
                  an XXXXX flow sensor.
									
									The output is displayed on a PC using the
									Serial Wire Viewer (SWV).  See the SWV examples
									for an explanation of the use of this feature.       

Requirements    : * STM32F4-Discovery Board
              
Circuit         : * A wire link between PB3 and pin6 of the SWD
										connector (CN3) as explained in the SWV
										examples section
									* The XXXXX flow sensor connected as:
										VCC			-		+5V
										SIGNAL	-		PE15	(pin is 5V tolerant)
										GND			-		GND

See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void EXTILine15_Config(void);																		//function prototype for a user funtion EXTILine15_Config()
void TIM14_Config(void);																				//function prototype for a user funtionTIM14_Config()

/* #global variables -----------------------------------------*/
__IO uint32_t pulse_counter=0, pulses_per_second=0;							//pulses_per_second will hold the latest result; pulse_counter is used by the interrupt routine



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
  
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config

	TIM14_Config();									//initialize TIM14 to generate an interrupt every 1 sec to measure the pulses per second
	EXTILine15_Config();						//initialize PE15 as a external interrupt pin to detect the pulses and count them						
	
	/* Main program loop */
  while (1)
  {
		printf("Pulses per second = %d\n", pulses_per_second);			//use the variable anywhere in the main code; it will always hold
																																//the latest result and will be updated every second automatically 
																																//using the TIM14 interrupt
		Delay(20);
  }	
}



/*
Retarget the C library printf function to the SWO Viewer.
Overwrites int fputc(int ch, FILE *f) function used by printf.
*/								
int fputc(int ch, FILE *f)
{
	return(ITM_SendChar(ch));
}



/* Initialize PE15 as a external interrupt pin to detect the pulses and count them */
void EXTILine15_Config(void)
{
  EXTI_InitTypeDef   EXTI_InitStructure;
	
  GPIO_InitTypeDef   GPIO_InitStructure;
  NVIC_InitTypeDef   NVIC_InitStructure;

  /* Enable GPIOE clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
	
  /* Enable SYSCFG clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
  
  /* Configure PE15 pin as input floating */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;			//can change this to GPIO_PuPd_UP or GPIO_PuPd_DOWN to use the internal Pull UP or DOWN resistors
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  /* Connect EXTI Line15 to PE15 pin */
  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE, EXTI_PinSource15);

  /* Configure EXTI Line15 */
  EXTI_InitStructure.EXTI_Line = EXTI_Line15;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;  
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  /* Enable and set EXTI Line15 Interrupt to the lowest priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}



/* Initialize TIM14 to generate an interrupt every 1 sec to measure the pulses per second */
void TIM14_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;													

  /* TIM14 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

  /* Enable the TIM14 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM8_TRG_COM_TIM14_IRQn;	
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = 8399;  								//PSC value; meanse that TIM register will be incremented every 100us (84000000 / (8399+1) = 10000Hz = 100us
  TIM_TimeBaseStructure.TIM_Period = 9999;                 		//ARR value - 1; create a time period of 1000000us (100us * (9999 + 1) = 1000000us or 1sec)
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM14, &TIM_TimeBaseStructure);

	/* TIM Interrupts enable */
  TIM_ITConfig(TIM14, TIM_IT_Update, ENABLE);

  /* TIM11 enable counter */
  TIM_Cmd(TIM14, ENABLE);
}



/* Interrupt routine for TIM14 which will be called every 1 second */
void TIM8_TRG_COM_TIM14_IRQHandler(void)
{
	if (TIM_GetITStatus(TIM14, TIM_IT_Update) != RESET)	//Make sure that interrupt flag is set
	{
		pulses_per_second = pulse_counter;						//transfer the latest count to pulses_per_second
		pulse_counter = 0;														//reset counter
		TIM_ClearITPendingBit(TIM14, TIM_IT_Update);	//clear interrupt flag
	}
}



/* Interrupt routine for PE15 which will be called everytime there is a rising edge on the pin */
void EXTI15_10_IRQHandler(void) 
{
  if (EXTI_GetITStatus(EXTI_Line15) != RESET) 		//Make sure that interrupt flag is set
	{
		pulse_counter++;															//increment counter
		EXTI_ClearITPendingBit(EXTI_Line15);					//clear interrupt flag
  }
}


