/*#############################################################
File Name       : ex2_DHT11_measure_interrupt_swv.c
Author          : Grant Phillips
Date Modified   : 15/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Example program that demonstrates the use of
                  a DHT11 Humidity-Temperature Sensor. In this
									example DHT11_Read() is automaticly called
									every second using a TIM14 interrupr.  The
									global variable DHT11_ExitCode is monitored
									to make the last result was valid.									
									
									The output is displayed on a PC using the
									Serial Wire Viewer (SWV).  See the SWV examples
									for an explanation of the use of this feature.

Requirements    : * STM32F4-Discovery Board
									* Make sure the DHT11_F4.c file is in the 
                    same folder than the project
              
Circuit         : * A wire link between PB3 and pin6 of the SWD
										connector (CN3) as explained in the SWV
										examples section
									* The DHT11 sensor connected as:
										1. VCC			-		3V
										2. DATA			-		PE15	(with a 5K6 pull-up 
																		resistor to 3V; pin is 5V tolerant)
										3. NC				-		Not connected
										4. GND			-		GND

See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision
#include "DHT11_F4.c"																						//for the DHT11 humidity-temperature sensor functions on the STM32F4-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void TIM14_Config(void);																				//function prototype for a user funtionTIM14_Config()

/* #global variables -----------------------------------------*/



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config

	DHT11_Init();																									//initialize the DHT11 humidity-temperature sensor
	TIM14_Config();																								//initialize TIM14 to generate an interrupt every 1 sec to read from the DHT11
	
	/* Main program loop */
  while (1)
  {
		if(DHT11_ExitCode == DHT11_OK)															//check if last results were OK
		{
			printf("Humidity = %d     Temperature = %d\n", DHT11_Humidity, DHT11_Temperature);	//display the results if result is OK
		}
		else if(DHT11_ExitCode == DHT11_ERROR_TIMEOUT)							//Check if last results caused errors.  These checks could be leaved out
		{																														
			printf("TIMEOUT error\n");
		}
		else if(DHT11_ExitCode == DHT11_ERROR_CHECKSUM)
		{
			printf("CHECKSUM error\n");
		}
  }	
}


/*
Retarget the C library printf function to the SWO Viewer.
Overwrites int fputc(int ch, FILE *f) function used by printf.
*/								
int fputc(int ch, FILE *f)
{
	return(ITM_SendChar(ch));
}



/* Initialize TIM14 to generate an interrupt every 1 sec to measure the pulses per second */
void TIM14_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;													

  /* TIM14 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

  /* Enable the TIM14 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM8_TRG_COM_TIM14_IRQn;	
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = 8399;  								//PSC value; meanse that TIM register will be incremented every 100us (84000000 / (8399+1) = 10000Hz = 100us
  TIM_TimeBaseStructure.TIM_Period = 9999;                 		//ARR value - 1; create a time period of 1000000us (100us * (9999 + 1) = 1000000us or 1sec)
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM14, &TIM_TimeBaseStructure);

	/* TIM Interrupts enable */
  TIM_ITConfig(TIM14, TIM_IT_Update, ENABLE);

  /* TIM11 enable counter */
  TIM_Cmd(TIM14, ENABLE);
}



/* Interrupt routine for TIM14 which will be called every 1 second */
void TIM8_TRG_COM_TIM14_IRQHandler(void)
{
	if (TIM_GetITStatus(TIM14, TIM_IT_Update) != RESET)	//Make sure that interrupt flag is set
	{
		DHT11_Read();																			//read from the DHT11 sensor
		TIM_ClearITPendingBit(TIM14, TIM_IT_Update);			//clear interrupt flag
	}
}
